//====================================================================================================
// DllLoader.cpp -   CDllLoader  CCommon
//====================================================================================================

#include "StdAfx.h"
#include "DllLoader.h"

//====================================================================================================
//  
//====================================================================================================
WSVECTOR CDllLoader::m_WSPlugins;  //     
HMODULEVEC CDllLoader::m_HModules;  //    
PIPLUGINVEC CDllLoader::m_PIPlugins;  //      

//====================================================================================================
// CDllLoader::LoadDll() -  dll    
//====================================================================================================
void CDllLoader::LoadDll(LPCWSTR pwsDllPath) {
  //  dll  
  HMODULE hDll = LoadLibrary(pwsDllPath);
  
  //  LPCWSTR  wstring
  wstring wsDllPath = pwsDllPath;

  // 
  if(!hDll) {
    wstring wsText = L"CDllLoader::LoadDll() :\n : \"" + wsDllPath + L"\"!";
    MessageBox(NULL, wsText.c_str(), L"", MB_OK);
    exit(1);  //  
	}  // if(!hDll)

  //    
  m_WSPlugins.push_back(wsDllPath);  //     

  //    
  m_HModules.push_back(hDll);  //    
}  // CDllLoader::LoadDll()

//====================================================================================================
// CDllLoader::LoadAllDll() -   dll   wsDllPathMask
//====================================================================================================
void CDllLoader::LoadAllDll(LPCWSTR pwsDllPathMask) {
  //       ,  
  if(!m_HModules.empty()) {
    MessageBox(NULL, L"  ,   !", L"", MB_OK);
    return;
  }  // if(!m_HModules.empty())

  //          
  WCHAR wcFullItemName[_MAX_PATH] = {0};
  
  //        
	WCHAR wcCurDirPathName[_MAX_DIR] = {0};

  //     
  _wgetcwd(wcCurDirPathName, _MAX_DIR);

	//        
	wsprintf(wcFullItemName, L"%s\\%s", wcCurDirPathName, pwsDllPathMask);

  //      
  WIN32_FIND_DATA FileData = {0};
  HANDLE FileHandle = FindFirstFile(wcFullItemName, &FileData);

  //    ,  
  if(FileHandle == INVALID_HANDLE_VALUE) return;

  //     ,     
  WCHAR wcDrive[_MAX_DRIVE] = {0};  //  
  WCHAR wcFName[_MAX_FNAME] = {0};
  WCHAR wcExt[_MAX_EXT] = {0};

  //   
  do {
    //      ()   (    )
    _wsplitpath_s(wcFullItemName, wcDrive, _MAX_DRIVE, wcCurDirPathName, _MAX_DIR, NULL, 0, wcExt, _MAX_EXT);  

    //    ,  
    _wsplitpath_s(FileData.cFileName, NULL, 0, NULL, 0, wcFName, _MAX_FNAME, NULL, 0);  

    //     ,  
    _wmakepath_s(wcFullItemName, wcDrive, wcCurDirPathName, wcFName, wcExt);

    //  dll    
	  LoadDll(wcFullItemName);
  } while(FindNextFile(FileHandle, &FileData));

  // Close the file when we are done
  FindClose(FileHandle);
}  // CDllLoader::LoadAllDll()

//====================================================================================================
// CDllLoader::UnloadDll() -  dll    
//====================================================================================================
void CDllLoader::UnloadDll(HMODULE hDll, wstring &wsDllPath) {
  //  hDll == NULL,   , !
	if(!hDll) return;

  // 
  if(!FreeLibrary(hDll)) {
    wstring wsText = L"CDllLoader::UnloadDll() :\n   :\n\"" + wsDllPath + L"\"!";
    MessageBox(NULL, wsText.c_str(), L"", MB_OK);
    exit(1);  //  
  }  // if(!FreeLibrary(hDll))
}  // CDllLoader::UnloadDll()

//====================================================================================================
// CDllLoader::UnloadAllDll() -    dll
//====================================================================================================
void CDllLoader::UnloadAllDll() {
  //       ,    dll
  while(!m_HModules.empty()) {
    HMODULE hDll = m_HModules.back();  //  
    wstring wsDllPath = m_WSPlugins.back();  //  

    //   dll    
    UnloadDll(hDll, wsDllPath);

    //     
    m_HModules.pop_back();
    m_WSPlugins.pop_back();
  }  // while(!m_HModules.empty())
}  // CDllLoader::UnloadAllDll()

//====================================================================================================
// CDllLoader::GetPluginInterface() -  , : CPluginInterface
//====================================================================================================
PIPLUGIN CDllLoader::GetPluginInterface(HMODULE hDll, wstring &wsDllPath) {
  //  

  //    GetPluginInterface()     hDll
  PGetPluginInterface pGetPluginInterface = (PGetPluginInterface) GetProcAddress(hDll, (LPCSTR) "GetPluginInterface");
		  
  //   
  if(!pGetPluginInterface) {
    wstring wsText = L"CDllLoader::GetPluginInterface() :\n     : \"" + 
        wsDllPath + L"\"!";
    MessageBox(NULL, wsText.c_str(), L"", MB_OK);
    exit(1);  //  !
  }  // if(!pGetPluginInterface)

  //      dll
  PIPLUGIN pPluginInterface = pGetPluginInterface();

  //   
  if(!pPluginInterface) {
    wstring wsText = L"CDllLoader::GetPluginInterface() :\n     :\n\"" + 
        wsDllPath + L"\"!";
    MessageBox(NULL, wsText.c_str(), L"", MB_OK);
    exit(1);  //  !
  }  // if(!m_pPluginInterface)

  //   
	return pPluginInterface;
}  // CDllLoader::GetPluginInterface()

//====================================================================================================
// CDllLoader::GetAllPluginInterfaces() -         
//====================================================================================================
PIPLUGINVEC &CDllLoader::GetAllPluginInterfaces() {  // LPCWSTR pwsGetInterface  // PIPLUGINVEC *pvPIPlugin
  //        ,  
  if(!m_PIPlugins.empty()) {
    MessageBox(NULL, L"    ,\n  !", L"", MB_OK);
    return m_PIPlugins;
  }  // if(!m_HModules.empty())

  ////       ,      
  //while(!m_HModules.empty()) {
  //  HMODULE hDll = m_HModules.back();  //  
  //  wstring wsDllPath = m_WSPlugins.back();  //  

  //  //    
  //  PIPLUGIN PIPlugin = GetPluginInterface(hDll, wsDllPath);

  //  //    
  //  m_PIPlugins.insert(m_PIPlugins.begin(), PIPlugin);
  //}  // while(!m_HModules.empty())

  for(size_t i = 0; i < m_HModules.size(); ++i) {
    HMODULE hDll = m_HModules.at(i);  //  
    wstring wsDllPath = m_WSPlugins.at(i);  //  

    //    
    PIPLUGIN PIPlugin = GetPluginInterface(hDll, wsDllPath);

    //    
    m_PIPlugins.push_back(PIPlugin);
  }  // for(size_t i = 0; i < m_HModules.size(); ++i)

  return m_PIPlugins;
}  // CDllLoader::GetAllPluginInterfaces()

//====================================================================================================
// GetDllLoaderInterface() -   ,     
//====================================================================================================
extern "C" DLL_API CDllLoaderInterface *GetDllLoaderInterface() {
  //    
  static CDllLoader *m_pDllLoaderInterface = new CDllLoader;  //     CCommon
	
  return m_pDllLoaderInterface;
}  // GetDllLoaderInterface()

//====================================================================================================
//====================================================================================================
